@@ -4,7 +4,7 @@
 '''
 
 from .omas_utils import *
-from .omas_core import ODS
+from .omas_core import ODS, ODC
 
 
 # ---------------------------
@@ -12,7 +12,7 @@ from .omas_core import ODS
 # ---------------------------
 def save_omas_json(ods, filename, objects_encode=None, **kw):
     """
-    Save an OMAS data set to Json
+    Save an ODS to Json
 
     :param ods: OMAS data set
 
@@ -42,9 +42,9 @@ def save_omas_json(ods, filename, objects_encode=None, **kw):
         f.write(json_string)
 
 
-def load_omas_json(filename, consistency_check=True, imas_version=omas_rcparams['default_imas_version'], **kw):
+def load_omas_json(filename, consistency_check=True, imas_version=omas_rcparams['default_imas_version'], cls=ODS, **kw):
     """
-    Load an OMAS data set from Json
+    Load ODS or ODC from Json
 
     :param filename: filename or file descriptor to load from
 
@@ -52,6 +52,8 @@ def load_omas_json(filename, consistency_check=True, imas_version=omas_rcparams[
 
     :param imas_version: imas version to use for consistency check
 
+    :param cls: class to use for loading the data
+
     :param kw: arguments passed to the json.loads mehtod
 
     :return: OMAS data set
@@ -59,22 +61,40 @@ def load_omas_json(filename, consistency_check=True, imas_version=omas_rcparams[
 
     printd('Loading OMAS data from Json: %s' % filename, topic='json')
 
-    def cls():
-        return ODS(imas_version=imas_version, consistency_check=False)
-
     if isinstance(filename, str):
         with open(filename, 'r') as f:
             json_string = f.read()
     else:
-        f = filename
-        json_string = f.read()
+        json_string = filename.read()
+
+    # allow for empty json file
+    if not len(json_string.strip()):
+        return ODS(imas_version=imas_version, consistency_check=consistency_check)
+
+    def base_class(x):
+        clsODS = lambda: ODS(imas_version=imas_version, consistency_check=False)
+        clscls = lambda: ODC(imas_version=imas_version, consistency_check=False)
+        try:
+            tmp = json_loader(x, clsODS, null_to=numpy.NaN)
+        except Exception:
+            tmp = json_loader(x, clscls, null_to=numpy.NaN)
+        return tmp
+
+    tmp = json.loads(json_string, object_pairs_hook=lambda x: base_class(x), **kw)
+
+    # convert to cls
+    tmp.__class__ = cls
+    data = tmp.omas_data
+    tmp.omas_data = OrderedDict()
+    tmp.update(data)
 
-    tmp = json.loads(json_string, object_pairs_hook=lambda x: json_loader(x, cls, null_to=numpy.NaN), **kw)
     # we must manually call set_child_locations since the json_loader
     # routine uses the ODS.setraw method that does not do that for us
     tmp.set_child_locations()
+
     # perform consistency check
     tmp.consistency_check = consistency_check
+
     return tmp
 
 
