@@ -4,7 +4,7 @@
 '''
 
 from .omas_utils import *
-from .omas_core import ODS, dynamic_ODS
+from .omas_core import ODS, ODC, dynamic_ODS
 
 
 # --------------------------------------------
@@ -12,7 +12,7 @@ from .omas_core import ODS, dynamic_ODS
 # --------------------------------------------
 def save_omas_nc(ods, filename, **kw):
     """
-    Save an OMAS data set to NetCDF file
+    Save an ODS to NetCDF file
 
     :param ods: OMAS data set
 
@@ -76,21 +76,25 @@ def get_ds_item(dataset, item):
     return tmp
 
 
-def load_omas_nc(filename, consistency_check=True):
+def load_omas_nc(filename, consistency_check=True, imas_version=omas_rcparams['default_imas_version'], cls=ODS):
     """
-    Load an OMAS data set from NetCDF file
+    Load ODS or ODC from NetCDF file
 
     :param filename: filename to load from
 
     :param consistency_check: verify that data is consistent with IMAS schema
 
+    :param imas_version: imas version to use for consistency check
+
+    :param cls: class to use for loading the data
+
     :return: OMAS data set
     """
     printd('Loading from %s' % filename, topic='nc')
 
     from netCDF4 import Dataset
 
-    ods = ODS(consistency_check=False)
+    ods = cls(imas_version=imas_version, consistency_check=False)
     with Dataset(filename, 'r') as dataset:
         for item in dataset.variables.keys():
             if item.endswith('_error_upper'):
