@@ -64,6 +64,65 @@ class constants(object):
     e = 1.6021766208e-19
 
 
+@add_to__ODS__
+def consistent_times(self, attempt_fix=True, raise_errors=True):
+    """
+    Assign .time and .ids_properties.homogeneous_time info for top-level structures
+    since these are required for writing an IDS to IMAS
+
+    :param attempt_fix: fix dataset_description and wall IDS to have 0 times if none is set
+
+    :param raise_errors: raise errors if could not satisfy IMAS requirements
+
+    :return: `True` if all is good, `False` if requirements are not satisfied, `None` if fixes were applied
+    """
+
+    # if called at top level, loop over all data structures
+    if not len(self.location):
+        out = []
+        for ds in self:
+            out.append(self.getraw(ds).physics_consistent_times(attempt_fix=attempt_fix, raise_errors=raise_errors))
+        if any([k is False for k in out]):
+            return False
+        elif any([k is None for k in out]):
+            return None
+        else:
+            return True
+
+    ds = p2l(self.location)[0]
+
+    extra_info = {}
+    time = self.time(extra_info=extra_info)
+    if extra_info['homogeneous_time'] is False:
+        self['ids_properties']['homogeneous_time'] = extra_info['homogeneous_time']
+    elif time is not None and len(time):
+        self['time'] = time
+        self['ids_properties']['homogeneous_time'] = extra_info['homogeneous_time']
+    elif attempt_fix and ds in ['dataset_description', 'wall']:
+        self['time'] = [0.0]
+        extra_info['homogeneous_time'] = True
+        self['ids_properties']['homogeneous_time'] = extra_info['homogeneous_time']
+        return None
+    elif raise_errors:
+        raise ValueError(self.location + '.time cannot be automatically filled! Missing time information in the data structure.')
+    else:
+        return False
+    return True
+
+
+@add_to__ODS__
+def imas_info(self):
+    # if called at top level, loop over all data structures
+    if not len(self.location):
+        for ds in self:
+            self.getraw(ds).physics_imas_info()
+        return
+    else:
+        self['ids_properties.version_put.access_layer'] = 'N/A'
+        self['ids_properties.version_put.access_layer_language'] = 'OMAS'
+        self['ids_properties.version_put.data_dictionary'] = self.imas_version
+
+
 @add_to__ODS__
 @preprocess_ods('equilibrium')
 def equilibrium_stored_energy(ods, update=True):
@@ -270,7 +329,8 @@ def summary_taue(ods, update=True):
 @preprocess_ods('core_sources')
 def summary_total_powers(ods, update=True):
     """
-    Integrate power densities to the total and heating and current drive systems and fills summary.globalquantities
+    Integrate power densities to the total and heating and current drive systems and fills summary.global_quantities
+
     :param ods: input ods
 
     :param update: operate in place
@@ -283,10 +343,13 @@ def summary_total_powers(ods, update=True):
 
         ods_n = ODS().copy_attrs_from(ods)
 
-    sources = ods_n['core_sources']['source']
+    if 'core_sources.source.0' not in ods_n:
+        return ods_n
+
+    sources = ods_n['core_sources.source']
     index_dict = {2: 'nbi', 3: 'ec', 4: 'lh', 5: 'ic'}
     power_dict = {'total': [], 'nbi': [], 'ec': [], 'lh': [], 'ic': []}
-    q_init = numpy.zeros(len(sources[0]['profiles_1d'][0]['grid']['rho_tor_norm']))
+    q_init = numpy.zeros(len(sources['0.profiles_1d.0.grid.volume']))
     q_dict = {
         'total': copy.deepcopy(q_init),
         'nbi': copy.deepcopy(q_init),
@@ -294,29 +357,25 @@ def summary_total_powers(ods, update=True):
         'lh': copy.deepcopy(q_init),
         'ic': copy.deepcopy(q_init),
     }
-
     ignore_indices = list(range(100, 108)) + list(range(900, 910))
 
-    for time_index in sources[0]['profiles_1d']:
-        vol = sources[0]['profiles_1d'][time_index]['grid']['volume']
+    for time_index in sources['0.profiles_1d']:
+        vol = sources[f'0.profiles_1d.{time_index}.grid.volume']
         for source in sources:
-            if sources[source]['identifier.index'] in ignore_indices:
+            if sources[f'{source}.identifier.index'] in ignore_indices:
                 # Skip the combined sources to prevent double counting
                 continue
-            if (
-                'electrons' in sources[source]['profiles_1d'][time_index]
-                and 'energy' in sources[source]['profiles_1d'][time_index]['electrons']
-            ):
-                q_dict['total'] += sources[source]['profiles_1d'][time_index]['electrons']['energy']
-                if sources[source]['identifier.index'] in index_dict:
-                    q_dict[index_dict[sources[source]['identifier.index']]] += sources[source]['profiles_1d'][time_index]['electrons'][
-                        'energy'
+            if f'{source}.profiles_1d.{time_index}.electrons.energy' in sources:
+                q_dict['total'] += sources[f'{source}.profiles_1d.{time_index}.electrons.energy']
+                if sources[f'{source}.identifier.index'] in index_dict:
+                    q_dict[index_dict[sources[f'{source}.identifier.index']]] += sources[
+                        f'{source}.profiles_1d.{time_index}.electrons.energy'
                     ]
-            if 'total_ion_energy' in sources[source]['profiles_1d'][time_index]:
-                q_dict['total'] += sources[source]['profiles_1d'][time_index]['total_ion_energy']
-                if sources[source]['identifier.index'] in index_dict:
-                    q_dict[index_dict[sources[source]['identifier.index']]] += sources[source]['profiles_1d'][time_index][
-                        'total_ion_energy'
+            if f'{source}.profiles_1d.{time_index}.total_ion_energy' in sources:
+                q_dict['total'] += sources[f'{source}.profiles_1d.{time_index}.total_ion_energy']
+                if sources[f'{source}.identifier.index'] in index_dict:
+                    q_dict[index_dict[sources[f'{source}.identifier.index']]] += sources[
+                        f'{source}.profiles_1d.{time_index}.total_ion_energy'
                     ]
 
         for key, value in power_dict.items():
@@ -339,6 +398,8 @@ def summary_global_quantities(ods, update=True):
     Calculates global quantities for each time slice and stores them in the summary ods:
      - Greenwald Fraction
      - Energy confinement time estimated from the IPB98(y,2) scaling
+     - Integrate power densities to the totals
+     - Generate summary.global_quantities from global_quantities of other IDSs
 
     :param ods: input ods
 
@@ -346,10 +407,62 @@ def summary_global_quantities(ods, update=True):
 
     :return: updated ods
     """
-    ods.physics_summary_greenwald()
-    ods.physics_summary_taue()
+    ods_n = ods.physics_summary_greenwald(update=update)
+    ods_n.physics_summary_taue(update=True)
+    ods_n.physics_summary_total_powers(update=True)
+    ods_n.physics_summary_consistent_global_quantities(update=True)
+    return ods_n
 
-    return ods
+
+@add_to__ODS__
+def summary_consistent_global_quantities(ods, ds=None, update=True):
+    """
+    Generate summary.global_quantities from global_quantities of other IDSs
+
+    :param ods: input ods
+
+    :param ds: IDS from which to update summary.global_quantities. All IDSs if `None`.
+
+    :param update: operate in place
+
+    :return: updated ods
+    """
+    ods_n = ods
+    if not update:
+        from omas import ODS
+
+        ods_n = ODS().copy_attrs_from(ods)
+
+    global_quantities = copy.copy(omas_global_quantities(ods.imas_version))
+
+    if ds is None:
+        ds = set(map(lambda x: x.split('.')[0], global_quantities))
+    ds = set(ds)
+    if 'summary' in ds:
+        ds.remove('summary')
+
+    # global_quantities destinations
+    dst = []
+    for item in global_quantities:
+        path = item.split('.')
+        if path[0] == 'summary':
+            dst.append(path[-1])
+
+    # global_quantities sources
+    src = []
+    for item in global_quantities:
+        path = item.split('.')
+        if path[0] in ds and path[0] in ods and path[-1] in dst:
+            src.append(item)
+
+    # copy global_quantities from other IDSs to summary
+    for item in src:
+        if item.replace(':', '0') in ods:
+            path = item.split('.')
+            ods_n[f'summary.global_quantities.{path[-1]}.value'] = ods[item]
+            ods_n[f'summary.global_quantities.{path[-1]}.source'] = 'Consistency with ' + path[0]
+
+    return ods_n
 
 
 @add_to__ODS__
@@ -1874,8 +1987,6 @@ _cocos_signals = {}
             with open(filename, 'r') as f:
                 original = str(f.read())
             new = str('\n'.join(csig))
-            import difflib
-
             diff = difflib.unified_diff(original, new)
             assert original == new, 'COCOS signals are not up-to-date! Run `make cocos` to update the omas_cocos.py file.\n' + ''.join(diff)
 
