@@ -116,81 +116,85 @@ def create_json_structure(imas_version=omas_rcparams['default_imas_version']):
         return inv
 
     def traverse(me, hout, path, fout, parent):
-        me = copy.copy(me)
-        hout_propagate = hout
-        path_propagate = copy.deepcopy(path)
-        parent = copy.deepcopy(parent)
+        try:
+            fname = None
+            me = copy.copy(me)
+            hout_propagate = hout
+            path_propagate = copy.deepcopy(path)
+            parent = copy.deepcopy(parent)
 
-        if '@structure_reference' in me and me['@structure_reference'] == 'self':
-            return hout, fout
+            if '@structure_reference' in me and me['@structure_reference'] == 'self':
+                return hout, fout
 
-        if '@name' in me:
-            name = me['@name']
-            path_propagate.append(name)
-            hout_propagate = {}
-            hout[name] = hout_propagate
-            # paths
-            if '@path_doc' in me:
-                fname = path_propagate[0] + '/' + me['@path_doc']
-                me['@full_path'] = path_propagate[0] + '/' + me['@path_doc']
-                del me['@path_doc']
+            if '@name' in me:
+                name = me['@name']
+                path_propagate.append(name)
+                hout_propagate = {}
+                hout[name] = hout_propagate
+                # paths
+                if '@path_doc' in me:
+                    fname = path_propagate[0] + '/' + me['@path_doc']
+                    me['@full_path'] = path_propagate[0] + '/' + me['@path_doc']
+                    del me['@path_doc']
+                else:
+                    fname = path_propagate[0]
+                    me['@full_path'] = path_propagate[0]
+                if '@path' in me:
+                    del me['@path']
+                # coordinates
+                for coord in [c for c in me if c.startswith('@coordinate')]:
+                    if '...' not in me[coord]:
+                        me[coord] = path_propagate[0] + '/' + me[coord]
+                # identifiers documentation
+                if '@doc_identifier' in me:
+                    doc_id = xmltodict.parse(open(imas_json_dir + '/../data-dictionary/' + me['@doc_identifier']).read())
+                    hlp = doc_id['constants']['int']
+                    doc = []
+                    for row in hlp:
+                        doc.append('%s) %s : %s' % (row['#text'], row['@name'], row['@description']))
+                    me['@documentation'] = me['@documentation'].strip() + '\n' + '\n'.join(doc)
+
+                fname = process_path(fname)
+                fout[fname] = {}
+
+            if isinstance(me, list):
+                keys = range(len(me))
             else:
-                fname = path_propagate[0]
-                me['@full_path'] = path_propagate[0]
-            if '@path' in me:
-                del me['@path']
-            # coordinates
-            for coord in [c for c in me if c.startswith('@coordinate')]:
-                if '...' not in me[coord]:
-                    me[coord] = path_propagate[0] + '/' + me[coord]
-            # identifiers documentation
-            if '@doc_identifier' in me:
-                doc_id = xmltodict.parse(open(imas_json_dir + '/../data-dictionary/' + me['@doc_identifier']).read())
-                hlp = doc_id['constants']['int']
-                doc = []
-                for row in hlp:
-                    doc.append('%s) %s : %s' % (row['#text'], row['@name'], row['@description']))
-                me['@documentation'] = me['@documentation'].strip() + '\n' + '\n'.join(doc)
+                keys = list(me.keys())
 
-            fname = process_path(fname)
-            fout[fname] = {}
+            if '@units' in me:
+                if fname == 'equilibrium.time_slice[:].constraints.q':  # bug fix for v3.18.0
+                    me['@units'] = '-'
+                if fname in [
+                    'equilibrium.time_slice[:].profiles_1d.geometric_axis.r',
+                    'equilibrium.time_slice[:].profiles_1d.geometric_axis.z',
+                ]:
+                    me['@coordinate'] = ['equilibrium.time_slice[:].profiles_1d.psi']
+                if me['@units'] in ['as_parent', 'as parent', 'as_parent_level_2']:
+                    me['@units'] = parent['units']
+                parent['units'] = me['@units']
 
-        if isinstance(me, list):
-            keys = range(len(me))
-        else:
-            keys = list(me.keys())
+            # children inherit lifecycle status from parent
+            if '@lifecycle_status' in me:
+                parent['lifecycle_status'] = me['@lifecycle_status']
+            elif parent['lifecycle_status'] and not isinstance(me, list):
+                me['@lifecycle_status'] = parent['lifecycle_status']
+                keys.append('@lifecycle_status')
 
-        if '@units' in me:
-            if fname == 'equilibrium.time_slice[:].constraints.q':  # bug fix for v3.18.0
-                me['@units'] = '-'
-            if fname in [
-                'equilibrium.time_slice[:].profiles_1d.geometric_axis.r',
-                'equilibrium.time_slice[:].profiles_1d.geometric_axis.z',
-            ]:
-                me['@coordinate'] = ['equilibrium.time_slice[:].profiles_1d.psi']
-            if me['@units'] in ['as_parent', 'as parent', 'as_parent_level_2']:
-                me['@units'] = parent['units']
-            parent['units'] = me['@units']
-
-        # children inherit lifecycle status from parent
-        if '@lifecycle_status' in me:
-            parent['lifecycle_status'] = me['@lifecycle_status']
-        elif parent['lifecycle_status'] and not isinstance(me, list):
-            me['@lifecycle_status'] = parent['lifecycle_status']
-            keys.append('@lifecycle_status')
-
-        is_leaf = True
-        for kid in keys:
-            if isinstance(me[kid], (dict, list)):
-                is_leaf = False
-                traverse(me[kid], hout_propagate, path_propagate, fout, parent)
-            elif kid not in ['@name', '@xmlns:fn']:
-                hout_propagate[kid] = me[kid]
-                fout[fname][kid] = me[kid]
-
-                # if is_leaf:
-                # print(path_propagate)
+            is_leaf = True
+            for kid in keys:
+                if isinstance(me[kid], (dict, list)):
+                    is_leaf = False
+                    traverse(me[kid], hout_propagate, path_propagate, fout, parent)
+                elif fname and fname in fout and kid not in ['@name', '@xmlns:fn']:
+                    hout_propagate[kid] = me[kid]
+                    fout[fname][kid] = me[kid]
 
+                    # if is_leaf:
+                    # print(path_propagate)
+        except Exception:
+            pprint(fout)
+            raise
         return hout, fout
 
     parent = {}
@@ -282,6 +286,13 @@ def create_json_structure(imas_version=omas_rcparams['default_imas_version']):
     dump_string = json.dumps(times, default=json_dumper, indent=1, separators=(',', ': '), sort_keys=True)
     open(imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '_times.json', 'w').write(dump_string)
 
+    # generate global_quantities cache file
+    global_quantities = extract_global_quantities(imas_version=imas_version)
+    dump_string = json.dumps(global_quantities, default=json_dumper, indent=1, separators=(',', ': '), sort_keys=True)
+    open(imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '_global_quantities.json', 'w').write(
+        dump_string
+    )
+
 
 def create_html_documentation(imas_version=omas_rcparams['default_imas_version']):
     filename = os.path.abspath(os.sep.join([imas_json_dir, imas_versions.get(imas_version, imas_version), 'omas_doc.html']))
@@ -374,7 +385,7 @@ def extract_coordinates(imas_version=omas_rcparams['default_imas_version']):
     from omas.omas_utils import list_structures
     from omas.omas_utils import load_structure
 
-    omas_coordinates = set()
+    coordinates = set()
     for structure in list_structures(imas_version=imas_version):
         tmp = load_structure(structure, imas_version)[0]
         coords = []
@@ -382,9 +393,9 @@ def extract_coordinates(imas_version=omas_rcparams['default_imas_version']):
             if 'coordinates' in tmp[item]:
                 coords.extend(map(i2o, tmp[item]['coordinates']))
         coords = list(filter(lambda x: '...' not in x, set(coords)))
-        omas_coordinates.update(coords)
+        coordinates.update(coords)
 
-    return sorted(list(omas_coordinates))
+    return sorted(list(coordinates))
 
 
 def extract_times(imas_version=omas_rcparams['default_imas_version']):
@@ -398,16 +409,41 @@ def extract_times(imas_version=omas_rcparams['default_imas_version']):
     from omas.omas_utils import list_structures
     from omas.omas_utils import load_structure
 
-    omas_times = []
+    times = []
     for structure in list_structures(imas_version=imas_version):
         tmp = load_structure(structure, imas_version)[0]
 
         for item in tmp:
             if not item.endswith('.time') or 'data_type' not in tmp[item] or tmp[item]['data_type'] == 'STRUCTURE':
                 continue
-            omas_times.append(item)
+            times.append(item)
 
-    return sorted(omas_times)
+    return sorted(times)
+
+
+def extract_global_quantities(imas_version=omas_rcparams['default_imas_version']):
+    """
+    return list of strings with .global_quantities across all structures
+
+    :param imas_version: imas version
+
+    :return: list with times
+    """
+    from omas.omas_utils import list_structures
+    from omas.omas_utils import load_structure
+
+    global_quantities = []
+    for structure in list_structures(imas_version=imas_version):
+        tmp = load_structure(structure, imas_version)[0]
+
+        for item in tmp:
+            if any([item.endswith(error) for error in ['_error_index', '_error_lower', '_error_upper']]):
+                continue
+            elif not item.endswith('.global_quantities.' + item.split('.')[-1]):
+                continue
+            global_quantities.append(item)
+
+    return sorted(global_quantities)
 
 
 def extract_ggd(imas_version=omas_rcparams['default_imas_version']):
