@@ -49,15 +49,19 @@ def different_ods(ods1, ods2, ignore_type=False, ignore_empty=False, prepend_pat
             elif not ignore_type and type(ods1[k]) != type(ods2[k]):
                 differences.append('DIFF: `%s` differ in type (%s,%s)' % ((prepend_path_string + k), type(ods1[k]), type(ods2[k])))
             elif numpy.atleast_1d(is_uncertain(ods1[k])).any() or numpy.atleast_1d(is_uncertain(ods2[k])).any():
-                if not numpy.allclose(nominal_values(ods1[k]), nominal_values(ods2[k]), equal_nan=True) or not numpy.allclose(
+                if nominal_values(ods1[k]).shape != nominal_values(ods2[k]).shape:
+                    differences.append('DIFF: `%s` differ in shape' % (prepend_path_string + k))
+                elif not numpy.allclose(nominal_values(ods1[k]), nominal_values(ods2[k]), equal_nan=True) or not numpy.allclose(
                     std_devs(ods1[k]), std_devs(ods2[k]), equal_nan=True
                 ):
                     differences.append('DIFF: `%s` differ in value' % (prepend_path_string + k))
             else:
-                if not numpy.allclose(ods1[k], ods2[k], equal_nan=True):
+                if nominal_values(ods1[k]).shape != nominal_values(ods2[k]).shape:
+                    differences.append('DIFF: `%s` differ in shape' % (prepend_path_string + k))
+                elif not numpy.allclose(ods1[k], ods2[k], equal_nan=True):
                     differences.append('DIFF: `%s` differ in value' % (prepend_path_string + k))
-        except Exception:
-            raise Exception(f'Error comparing {k}')
+        except Exception as _excp:
+            raise Exception(f'Error comparing {k}: ' + repr(_excp))
     if len(differences):
         return differences
     else:
@@ -552,6 +556,8 @@ _info_structures = {}
 _coordinates = {}
 # dictionary that contains all the times defined within the data dictionary
 _times = {}
+# dictionary that contains all the _global_quantities defined within the data dictionary
+_global_quantities = {}
 
 # extra structures that python modules using omas can define
 # by setting omas.omas_utils._extra_structures equal to a
@@ -695,6 +701,27 @@ def omas_times(imas_version=omas_rcparams['default_imas_version']):
     return _times[imas_version]
 
 
+def omas_global_quantities(imas_version=omas_rcparams['default_imas_version']):
+    """
+    return list of times
+
+    :param imas_version: IMAS version to look up
+
+    :return: list of strings with IMAS times
+    """
+    # caching
+    if imas_version not in _global_quantities:
+        filename = imas_json_dir + os.sep + imas_versions.get(imas_version, imas_version) + os.sep + '_global_quantities.json'
+        if os.path.exists(filename):
+            with open(filename, 'r') as f:
+                _global_quantities[imas_version] = json.load(f)
+        else:
+            from .omas_structure import extract_global_quantities
+
+            _global_quantities[imas_version] = extract_global_quantities(imas_version)
+    return _global_quantities[imas_version]
+
+
 _p2l_cache = {}
 
 
@@ -756,10 +783,12 @@ def l2i(path):
 
     :return: IMAS path format
     """
-    ipath = path[0]
-    for step in path[1:]:
+    ipath = ''
+    for kstep, step in enumerate(path):
         if isinstance(step, int) or step == ':':
             ipath += "[%s]" % step
+        elif kstep == 0:
+            ipath += '%s' % step
         else:
             ipath += '.%s' % step
     return ipath
