@@ -127,6 +127,22 @@ class TestOmasCore(unittest.TestCase):
         )
         return
 
+    def test_data_slicing(self):
+        ods = ODS()
+        ods['langmuir_probes.embedded.0.name'] = '1'
+        ods['langmuir_probes.embedded.1.name'] = '12'
+        ods['langmuir_probes.embedded.2.name'] = '123'
+        assert ods['langmuir_probes']['embedded.:.name'][2] == '123'
+
+        ods = ODS()
+        for k in range(3):
+            ods[f'langmuir_probes.embedded.{k}.time'] = [float(k)] * (k + 1)
+        assert numpy.allclose(
+            ods['langmuir_probes']['embedded.:.time'],
+            numpy.array([[0.0, numpy.nan, numpy.nan], [1.0, 1.0, numpy.nan], [2.0, 2.0, 2.0]]),
+            equal_nan=True,
+        )
+
     def test_dynamic_set_nonzero_array_index(self):
         ods = ODS()
         ods.consistency_check = False
@@ -427,6 +443,39 @@ class TestOmasCore(unittest.TestCase):
         assert len(ods['ec_launchers.code.parameters.launcher.0']) == 2
         return
 
+    def test_latexit(self):
+        assert latexit['somewhere.:.sublocation.n_e'] == '$n_e$'
+        assert latexit['.n_e'] == '$n_e$'
+        assert latexit['n_e'] == '$n_e$'
+
+        assert latexit['core_profiles.profiles_1d[:].electrons.density_thermal'] == '$n_e$'
+        assert latexit['barometry.gauge[:].pressure.data'] == '$P$'
+        assert latexit['equilibrium.time_slice[0].ggd[1].b_field_tor[0].values'] == r'$B_{\phi}$'
+        assert latexit['core_profiles.profiles_1d[4].ion[0].density'] == '$n_{i0}$'
+
+        try:
+            latexit['somewhere.does_not_exist']
+            raise RuntimeError('latexit of missing variable should fail')
+        except KeyError:
+            pass
+
+        assert latexit.get('somewhere.does_not_exist', 'somewhere.does_not_exist') == 'somewhere.does_not_exist'
+
+    def test_odc(self):
+        odc = ODC()
+        for k in range(5):
+            odc[f'133221.equilibrium.time_slice.{k}.global_quantities.ip'] = 1000.0 + k + 1
+            odc[f'133229.equilibrium.time_slice.{k}.global_quantities.ip'] = 2000.0 + k + 1
+            odc[f'133230.equilibrium.time_slice.{k}.global_quantities.ip'] = 3000.0 + k + 1
+        assert odc.keys() == [133221, 133229, 133230]
+        assert odc[':.equilibrium.time_slice.:.global_quantities.ip'].size == 15
+
+        for ftype in ['h5', 'pkl', 'nc', 'json']:
+            odc.save('test.' + ftype)
+            odc1 = ODC().load('test.' + ftype)
+            diff = odc1.diff(odc)
+            assert not diff, f'save/load ODC to {ftype} failed:\r{repr(diff)}'
+
     # End of TestOmasCore class
 
 
